/*
ARC385 A2: The Useless Machine
Part 2: Joystick and Servo Motor

Written by: Songzi Zhou, Minh (Camilla) Hoang, Jasmin Kochar, Boer Li, Alice Niu

03-24-2024

This code was written with thanks and reference to:
https://www.arduino.cc/reference/en/language/functions/random-numbers/random/;
https://arduinogetstarted.com/tutorials/arduino-joystick
https://www.makerguides.com/servo-arduino-tutorial/
*/


//buzzer
const int buzzer = 11;                                   //attach buzzer to pin 11

//set buzzer frequency
int frequencyH = 800;                  
int frequencyL = 500;

//servo motor
#include <Servo.h>                                       //include the servo library
Servo pbj;                                               //create a new servo object
#define servoPin 3                                       //attach servo motor to pin 3

//rotation angles for servo motor 
int angleWater = 180;                                    //relative position of water level sensor in degrees
int angleUltra = 82;                                     //relative position of ultrasonic sensor in degrees
int anglePhoto = 41;                                     //relative position of photoresistor in degrees
int angleStandby = 7;                                    //relative standby position for the servo motor in degrees
int shake = 7;                                           //degree of rotation of "headshake"

//neutral readings for joystick calibration
int xValueNeutral;
int yValueNeutral;

//joystick
int joystickX = A0;                                      //define joystick X-value pin
int joystickY = A1;                                      //define joystick Y-value pin
int xValue = xValueNeutral;
int yValue = xValueNeutral;
int deadzone = 100;                                      //set central area as non-responsive 

//delays
int pause = 500;                                         //set duration of servo motor delay
int nono = 125;                                          //set duration of "headshake" 

//count #
int myCounter = 0;                                       //keep track of time 


void setup() {
  Serial.begin(9600);
  pbj.attach(servoPin); 
  
  //buzzer
  pinMode(buzzer, OUTPUT);  

  //read neutral position of joystick                             
  xValueNeutral = analogRead (joystickX);                                     
  yValueNeutral = analogRead (joystickY);
}


void loop() {

  //set print conditions to false
  static bool conditionA_printed = false;
  static bool conditionB_printed = false;
  static bool conditionC_printed = false;
  static bool conditionD_printed = false;

  //condition A
  while (myCounter >= 0 && myCounter < 60) {                                  //condition for the first 30 seconds

    //read values of joystick
    xValue = analogRead (joystickX);
    yValue = analogRead (joystickY);

    //print "Condition A" only once
    if (!conditionA_printed) {
    Serial.println();
    Serial.println("Condition A");
    conditionA_printed = true;                                                //set flag to true after printing
    }

    //print count #
    Serial.print("\t");
    Serial.print("Count # ");
    Serial.print(myCounter);
    Serial.print("\t");
    Serial.print("\t");

    //print values of joystick
    Serial.print("Joystick Coordinates: (");
    Serial.print(xValue);
    Serial.print(", ");
    Serial.print(yValue);
    Serial.print(")");
    Serial.print("\t");
    Serial.print("\t");

    //divide joystick motion into 4 quadrants
    //quadrant 4: up and right
    if (xValue - xValueNeutral > deadzone && yValue - yValueNeutral > deadzone) {
    Serial.println("Quadrant 4");
    pbj.write(angleWater);
    delay(pause);
    } 
    //quadrant 3: up and left
      else if (xValueNeutral - xValue > deadzone && yValue - yValueNeutral > deadzone) {
      Serial.println("Quadrant 3");
      pbj.write(angleUltra);
      delay(pause);
      } 
        //quadrant 2: down and left
        else if (xValueNeutral - xValue > deadzone && yValueNeutral - yValue > deadzone) {
        Serial.println("Quadrant 2");
        pbj.write(anglePhoto);
        delay(pause);
      } 
        //quadrant 1: down and right
        else if (xValue - xValueNeutral > deadzone && yValueNeutral - yValue > deadzone) {
          Serial.println("Quadrant 1");
          tone(buzzer, frequencyH);                                             //buzzer sound signals "wrong" answer
          pbj.write(angleStandby-shake);                                        //pointer movement mimics a headshake
          delay(nono);
          tone(buzzer, frequencyL);
          pbj.write(angleStandby+shake);
          delay(nono);
          tone(buzzer, frequencyH);
          pbj.write(angleStandby-shake);
          delay(nono);
          tone(buzzer, frequencyL);
          pbj.write(angleStandby+shake);
          delay(nono);
          noTone(buzzer);
          } else {
            Serial.println("Deadzone");
            delay(pause);
            }
    myCounter = myCounter + 1;  
  }

  //condition B
  while (myCounter >= 60 && myCounter < 120) {                                  //swtiches to condition B after 30 seconds

    //read values of joystick
    xValue = analogRead (joystickX);
    yValue = analogRead (joystickY);

    //print "Condition B" only once
    if (!conditionB_printed) {
    Serial.println();
    Serial.println("Condition B");
    conditionB_printed = true;                                                  //set flag to true after printing
    }

    //print count #
    Serial.print("\t");
    Serial.print("Count # ");
    Serial.print(myCounter);
    Serial.print("\t");
    Serial.print("\t");

    //print values of joystick
    Serial.print("Joystick Coordinates: (");
    Serial.print(xValue);
    Serial.print(", ");
    Serial.print(yValue);
    Serial.print(")");
    Serial.print("\t");
    Serial.print("\t");

    //divide joystick motion into 4 quadrants
    //quadrant 4: up and right
    if (xValue - xValueNeutral > deadzone && yValue - yValueNeutral > deadzone) {
    Serial.println("Quadrant 4");
    pbj.write(angleUltra);
    delay(pause);
    } 
      //quadrant 3: up and left
      else if (xValueNeutral - xValue > deadzone && yValue - yValueNeutral > deadzone) {
      Serial.println("Quadrant 3");
      pbj.write(anglePhoto);
      delay(pause);
      } 
        //quadrant 2: down and left
        else if (xValueNeutral - xValue > deadzone && yValueNeutral - yValue > deadzone) {
        Serial.println("Quadrant 2");
        tone(buzzer, frequencyH);                                               //buzzer sound signals "wrong" answer
        pbj.write(angleStandby-shake);                                          //pointer movement mimics a headshake
        delay(nono);
        tone(buzzer, frequencyL);
        pbj.write(angleStandby+shake);
        delay(nono);
        tone(buzzer, frequencyH);                                             
        pbj.write(angleStandby-shake);
        delay(nono);
        tone(buzzer, frequencyL);
        pbj.write(angleStandby+shake);
        delay(nono);
        noTone(buzzer);
      } 
        //quadrant 1: down and right
        else if (xValue - xValueNeutral > deadzone && yValueNeutral - yValue > deadzone) {
        Serial.println("Quadrant 1");
        pbj.write(angleWater);
        delay(pause);
        } else {
          Serial.println("Deadzone");
          delay(pause);
          }
      myCounter = myCounter + 1;  
  }

  //condition C
  while (myCounter >= 120 && myCounter < 180) {                                 //swtiches to condition C after 60 seconds

    //read values of joystick
    xValue = analogRead (joystickX);
    yValue = analogRead (joystickY);

    //print "Condition C" only once
    if (!conditionC_printed) {
    Serial.println();
    Serial.println("Condition C");
    conditionC_printed = true;                                                  //set flag to true after printing
    }

    //print count #
    Serial.print("\t");
    Serial.print("Count # ");
    Serial.print(myCounter);
    Serial.print("\t");
    Serial.print("\t");

    //print values of joystick
    Serial.print("Joystick Coordinates: (");
    Serial.print(xValue);
    Serial.print(", ");
    Serial.print(yValue);
    Serial.print(")");
    Serial.print("\t");
    Serial.print("\t");

    //divide joystick motion into 4 quadrants
    //quadrant 4: up and right
    if (xValue - xValueNeutral > deadzone && yValue - yValueNeutral > deadzone) {
    Serial.println("Quadrant 4");
    pbj.write(anglePhoto);
    delay(pause);
    } 
      //quadrant 3: up and left
      else if (xValueNeutral - xValue > deadzone && yValue - yValueNeutral > deadzone) {
      Serial.println("Quadrant 3");
      tone(buzzer, frequencyH);                                                 //buzzer sound signals "wrong" answer
      pbj.write(angleStandby-shake);                                            //pointer movement mimics a headshake
      delay(nono);
      tone(buzzer, frequencyL);                                         
      pbj.write(angleStandby+shake);
      delay(nono);
      tone(buzzer, frequencyH);                          
      pbj.write(angleStandby-shake);
      delay(nono);
      tone(buzzer, frequencyL);
      pbj.write(angleStandby+shake);
      delay(nono);
      noTone(buzzer);
      } 
        //quadrant 2: down and left
        else if (xValueNeutral - xValue > deadzone && yValueNeutral - yValue > deadzone) {
        Serial.println("Quadrant 2");
        pbj.write(angleWater);
        delay(pause);
        } 
          //quadrant 1: down and right
          else if (xValue - xValueNeutral > deadzone && yValueNeutral - yValue > deadzone) {
          Serial.println("Quadrant 1");
          pbj.write(angleUltra);
          delay(pause);
          } else {
            Serial.println("Deadzone");
            delay(pause);
            }
    myCounter = myCounter + 1;  
  }

  //condition D
  while (myCounter >= 180 && myCounter < 240) {                                 //swtiches to condition D after 90 seconds

    //read values of joystick
    xValue = analogRead (joystickX);
    yValue = analogRead (joystickY);

    //print "Condition D" only once
    if (!conditionD_printed) {
    Serial.println();
    Serial.println("Condition D");
    conditionD_printed = true;   
    }

    //print count #
    Serial.print("\t");
    Serial.print("Count # ");
    Serial.print(myCounter);
    Serial.print("\t");
    Serial.print("\t");

    //print values of joystick
    Serial.print("Joystick Coordinates: (");
    Serial.print(xValue);
    Serial.print(", ");
    Serial.print(yValue);
    Serial.print(")");
    Serial.print("\t");
    Serial.print("\t");

    //divide joystick motion into 4 quadrants
    //quadrant 4: up and right
    if (xValue - xValueNeutral > deadzone && yValue - yValueNeutral > deadzone) {
    Serial.println("Quadrant 4");
    tone(buzzer, frequencyH);                                                   //buzzer sound signals "wrong" answer
    pbj.write(angleStandby-shake);                                              //pointer movement mimics a headshake
    delay(nono);
    tone(buzzer, frequencyL);                                            
    pbj.write(angleStandby+shake);
    delay(nono);
    tone(buzzer, frequencyH);                                            
    pbj.write(angleStandby-shake);
    delay(nono);
    tone(buzzer, frequencyL);
    pbj.write(angleStandby+shake);
    delay(nono);
    noTone(buzzer);
    } 
      //quadrant 3: up and left
      else if (xValueNeutral - xValue > deadzone && yValue - yValueNeutral > deadzone) {
      Serial.println("Quadrant 3");
      pbj.write(angleWater);
      delay(pause);
      } 
        //quadrant 2: down and left
        else if (xValueNeutral - xValue > deadzone && yValueNeutral - yValue > deadzone) {
        Serial.println("Quadrant 2");
        pbj.write(angleUltra);
        delay(pause);
        } 
          //quadrant 1: down and right
          else if (xValue - xValueNeutral > deadzone && yValueNeutral - yValue > deadzone) {
          Serial.println("Quadrant 1");
          pbj.write(anglePhoto);
          delay(pause);
          } else {
            Serial.println("Deadzone");
            delay(pause);
            }
    myCounter = myCounter + 1;  
  }

  //reset the puzzle after 120 seconds
    if (myCounter >= 240) {
    myCounter = 0;                                                               
    
    //reset the game to condition A
    Serial.println();
    Serial.println();
    Serial.println("GAME RESET");
    Serial.println();
    Serial.println();
    }
  
}